from scipy import sparse as sparse
import numpy as np
import pickle as pkl
import time
import os

from HARE.Utility.config import data_dir

root_dir = os.getcwd()


def hare(name, dsname, epsilon, damping, saveresults=True, printerror=False, printruntimes=False):
    if ".ttl" in name: name = name[:-4]
    if ".nt" in name: name = name[:-3]
    
    load_dir = data_dir + "Matrices/{}/".format(dsname)
    save_dir = data_dir + "Results/{}/".format(dsname)
    
    #save_dir_alt =os.path.join(root_dir, "data_inputs/") + "hare/{}/".format(dsname)
    
    tic = time.time()
    print("LOADING F")
    y = np.load(load_dir + "F_" + name + ".npz")
    F = sparse.coo_matrix((y['data'],(y['row'],y['col'])),shape=y['shape'])
    
    print("LOADING W")
    y = np.load(load_dir + "W_" + name + ".npz")
    W = sparse.coo_matrix((y['data'],(y['row'],y['col'])),shape=y['shape'])
    
    print("CALCULATING P_N")
    P = sparse.csr_matrix(F.dot(W))
    n = P.shape[0]
    
    previous = np.ones(n)/n
    ones = np.ones(n)/n
    
    error = 1
    tic2 = time.time()
    
    while error > epsilon:
        tmp = np.array(previous)
        previous = damping*P.T.dot(previous) + (1-damping)*ones
        error = np.linalg.norm(tmp - previous)
        if(printerror):
            print(error)
            
    resourcedistribution = previous
    tripledistribution = F.T.dot(previous)
    tac = time.time()
    runtime = tac-tic
    runtime2 = tac-tic2
    if(printruntimes):
        print("RUNTIME with load: ", runtime)
        print("RUNTIME without load: ", runtime2)
    if(saveresults):
        print("LOADING DICTIONARIES")
        E2I = pkl.load(open(load_dir + "e2i_" + name + ".pkl", "rb"))
        T2I = pkl.load(open(load_dir + "t2i_" + name + ".pkl", "rb"))
        P2I = pkl.load(open(load_dir + "p2i_" + name + ".pkl", "rb"))
        I2E = dict()
        I2T = dict()
        I2P = dict()
        for key, val in E2I.items():
            I2E[val[0]] = key
        for key, val in T2I.items():
            I2T[val] = key
        for key, val in P2I.items():
            I2P[val[0]] = key
            
        print("##### start resource distribution ##########")
        print(resourcedistribution)
        print("##### end resource distribution ##########")

        with open(save_dir + "results_resources_" + name + "_HARE.txt", "w") as results:
            for i, x in sorted(enumerate(resourcedistribution), key = lambda x: -x[1]):
                tmp = ""		
                tmp += str(I2E[i])
                #print("#### start I2E #####")
                #print(tmp)
                #print("#### end I2E #####")
                results.write("{}\t{}\n".format(tmp, str(x)))

        with open(save_dir + "results_predicates_" + name + "_HARE.txt", "w") as results:
            for i, x in sorted(enumerate(resourcedistribution), key = lambda x: -x[1]):
                if i in I2P:
                    tmp = ""        
                    tmp += str(I2P[i])
                    print("#### start I2P #####")
                    print(tmp)
                    print("#### end I2P #####")
                    results.write("{}\t{}\n".format(tmp, str(x)))
        
        with open(save_dir + "results_triples_" + name + "_HARE.txt", "w") as results:
            for i, x in sorted(enumerate(tripledistribution), key = lambda x: -x[1]):
                tmp = ""		
                for n in I2T[i]:
                    tmp += str(n) + " "
                results.write("{}\t{}\n".format(tmp, str(x)))
    return runtime2
