from scipy import sparse as sparse
import numpy as np
import pickle as pkl
import time
import os

from HARE.Utility.config import data_dir

root_dir = os.getcwd()

def pagerank(name, dsname, epsilon, damping, saveresults=True, printerror=False, printruntimes=False):
    save_dir_alt =os.path.join(root_dir, "data_inputs/") + "pagerank/{}/".format(dsname)
    load_dir = data_dir + "Matrices/{}/".format(dsname)
    save_dir = data_dir + "Results/{}/".format(dsname)
    
    if ".ttl" in name: name = name[:-4]
    if ".nt" in name: name = name[:-3]
    
    tic = time.time()
    print("LOADING F")
    y = np.load(load_dir + "F_" + name + ".npz")
    F = sparse.coo_matrix((y['data'],(y['row'],y['col'])),shape=y['shape'])
    
    print("LOADING W")
    y = np.load(load_dir + "W_" + name + ".npz")
    W = sparse.coo_matrix((y['data'],(y['row'],y['col'])),shape=y['shape'])
    
    print("CALCULATING P_N")
    P = sparse.bmat([[None, W], [F, None]])
    n = P.shape[0]
    
    previous = np.ones(n)/n
    ones = np.ones(n)/n
    
    error = 1
    tic2 = time.time()
    
    while error > epsilon:
        tmp = np.array(previous)
        previous = damping*P.T.dot(previous) + (1-damping)*ones
        error = np.linalg.norm(tmp - previous)
        if(printerror):
            print(error)
            
    tac = time.time()
    runtime = tac-tic
    runtime2 = tac-tic2
    
    if(printruntimes):
        print("RUNTIME with loading: ", runtime)
        print("RUNTIME without loading: ", runtime2)
    
    if(saveresults):
        print("LOADING DICTIONARIES")
        E2I = pkl.load(open(load_dir + "e2i_" + name + ".pkl", "rb"))
        T2I = pkl.load(open(load_dir + "t2i_" + name + ".pkl", "rb"))
        I2E = dict()
        I2T = dict()
        
        for key, val in E2I.items():
            I2E[val[0]] = key
        
        for key, val in T2I.items():
            I2T[val] = key
        
        print("WRITING RESULTS")
        with open(save_dir_alt + "results_resources_" + name + "_PAGERANK.txt", "w") as results:
            results.write("")    
        with open(save_dir_alt + "results_triples_" + name + "_PAGERANK.txt", "w") as results_t:
            results_t.write("")
		
        with open(save_dir_alt + "results_" + name + "_PAGERANK.txt", "w") as results:
            for i, x in sorted(enumerate(previous), key = lambda x: -x[1]):
                tmp = ""
                if i < len(I2T):		
                    tmp += str(I2T[i])
                    results.write(tmp +  " " + str(x) + "\n")
                    with open(save_dir_alt + "results_triples_" + name + "_PAGERANK.txt", "a") as results_t:
                        results_t.write("{}\t{}\n".format(tmp,str(x)))
                else:
                    tmp += str(I2E[i- len(I2T)])
                    results.write(tmp + " " + str(x) + "\n")
                    with open(save_dir_alt + "results_resources_" + name + "_PAGERANK.txt", "a") as results_e:
                        results_e.write("{}\t{}\n".format(tmp,str(x)))
    return runtime
